<?php

namespace Drupal\trinion_tp\Controller;

use Dompdf\Dompdf;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\AlertCommand;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\node\Entity\Node;

/**
 * Утверждение или отмена документа
 */
class UtverzhdenieDokumenta extends ControllerBase {

  /**
   * Builds the response.
   * @param $op 0 - аннулирован, 1 - зарезервирован (+утвержден), 2 - утвержден
   */
  public function build(Node $node, $op) {
    $response = new AjaxResponse();
    $bundle = $node->bundle();
    $error = FALSE;
    $transaction = \Drupal::database()->startTransaction();
    if ($bundle == 'postuplenie_tovarov') {
      $sklad = $node->get('field_tp_sklad')->getString();
      if (empty($sklad)) {
        $error = TRUE;
        \Drupal::messenger()->addWarning('Поле Склад не заполнено');
      }
    }
    elseif ($bundle == 'zakaz_klienta') {
      foreach ($node->get('field_tp_stroki') as $stroka) {
        $stroka = $stroka->entity;
        $sklad_tid = $stroka->get('field_tp_sklad')->getString();
        $tovar = $stroka->get('field_tp_tovar')->first()->entity;
        if (!$sklad_tid) {
          $error = TRUE;
          \Drupal::messenger()->addWarning('У товара ' . $tovar->label() . ' поле Склад не заполнено');
        }
        elseif ($op == 1) {
          $edinica_izmereniya_tid = $stroka->get('field_tp_edinica_izmereniya')->getString();
          $harakteristika_tid = $stroka->get('field_tp_kharakteristika_tovara')->getString();
          $jivoy_ostatok = \Drupal::service('trinion_tp.helper')->getLiveOstatok($tovar->id(), $sklad_tid, $harakteristika_tid, $edinica_izmereniya_tid);
          $kolichestvo = $stroka->get('field_tp_kolichestvo')->getString();
          if ($jivoy_ostatok < $kolichestvo) {
            $error = TRUE;
            \Drupal::messenger()->addWarning('У товара ' . $tovar->label() . ' живой остаток на складе ' . $stroka->get('field_tp_sklad')->first()->entity->label() . ' меньше заказа. На складе в живом остатке ' . $jivoy_ostatok);
          }
        }
      }
    }
    elseif ($bundle == 'otgruzka') {
      $sklad_tid = $node->get('field_tp_sklad')->getString();
      if (empty($sklad_tid)) {
        $error = TRUE;
        \Drupal::messenger()->addWarning('Поле Склад не заполнено');
      }
//      else {
//        foreach ($node->get('field_tp_stroki') as $stroka) {
//          $stroka = $stroka->entity;
//          $tovar = $stroka->get('field_tp_tovar')->first()->entity;
//          if ($op == 1) {
//            $edinica_izmereniya_tid = $stroka->get('field_tp_edinica_izmereniya')->getString();
//            $harakteristika_tid = $stroka->get('field_tp_kharakteristika_tovara')->getString();
//            $jivoy_ostatok = \Drupal::service('trinion_tp.helper')->getLiveOstatok($tovar->id(), $sklad_tid, $harakteristika_tid, $edinica_izmereniya_tid);
//            $kolichestvo = $stroka->get('field_tp_kolichestvo')->getString();
//            if ($jivoy_ostatok < $kolichestvo) {
//              $error = TRUE;
//              \Drupal::messenger()->addWarning('У товара ' . $tovar->label() . ' живой остаток на складе ' . $node->get('field_tp_sklad')->first()->entity->label() . ' меньше заказа. На складе в живом остатке ' . $jivoy_ostatok);
//            }
//          }
//        }
//      }
    }

    if (!$error) {
      if ($bundle == 'postuplenie_tovarov') {
        $node->field_tp_utverzhdeno = $op;
        $this->utverzhdeniePostupleniyaTovarov($node, $op);
        \Drupal::messenger()->addStatus('Остатки на складе ' . ($op ? 'увеличены' : 'уменьшены'));
      } elseif ($bundle == 'zakaz_klienta') {
        $this->utverzhdenieZakazaKlienta($node, $op);
        if ($op == 0) {
          $node->field_tp_utverzhdeno = 0;
          $node->field_tp_zarezervirovan = 0;
          \Drupal::messenger()->addStatus('Резерв товаров удален');
        }
        elseif ($op == 1) {
          $node->field_tp_utverzhdeno = 1;
          $node->field_tp_zarezervirovan = 1;
          \Drupal::messenger()->addStatus('Резерв товаров создан');
          if ($node->get('field_tp_zakaz_dlya')->first()->entity->get('field_tl_roznichnyy_pokupatel')->getString()) {
            $vigruzka = new FrontolVigruzkaZakazov();
            $vigruzka->createOrderFile($node);
            \Drupal::messenger()->addStatus('Заказ выгружен во фронтол, через несколько секунд он там появится.');
          }
        }
        elseif ($op == 2) {
          $node->field_tp_utverzhdeno = 1;
          \Drupal::messenger()->addStatus('Заказ утвержден');
        }
      }
      elseif ($bundle == 'otgruzka') {
        $node->field_tp_utverzhdeno = $op;
        $this->utverzhdenieOtgruzki($node, $op);
      }
      elseif ($bundle == 'kommercheskoe_predlozhenie') {
        $node->field_tp_utverzhdeno = $op;
      }
      elseif ($bundle == 'zakaz_postavschiku') {
        $node->field_tp_utverzhdeno = $op;
      }
      elseif ($bundle == 'schet') {
        $node->field_tp_utverzhdeno = $op;
      }
      elseif ($bundle == 'schet_postavschika') {
        $node->field_tp_utverzhdeno = $op;
      }
      elseif ($bundle == 'poluchennyy_platezh') {
        $node->field_tp_utverzhdeno = $op;
      }
      elseif ($bundle == 'otpravlennyy_platezh') {
        $node->field_tp_utverzhdeno = $op;
      }

      $node->save();
    }
    $response->addCommand(new RedirectCommand('/node/' . $node->id()));
    unset($transaction);
    return $response;
  }

  /**
   * Обработка утверждения/аннулирования поступления товаров
   * @param Node $node
   */
  public function utverzhdeniePostupleniyaTovarov(Node $node, $op) {
    foreach ($node->get('field_tp_stroki') as $item) {
      if ($tovar = $item->entity->get('field_tp_tovar')->first()) {
        $tovar_nid = $tovar->entity->id();
        $sklad = $node->get('field_tp_sklad')->getString();
        $edinica_izereniya_tid = $item->entity->get('field_tp_edinica_izmereniya')->getString();
        $harakteristika_tid = $item->entity->get('field_tp_kharakteristika_tovara')->getString();
        $ostatok = \Drupal::service('trinion_tp.helper')->getOstatokTovaraNode($tovar_nid, $sklad, $node->id(), $harakteristika_tid, $edinica_izereniya_tid, TRUE);
        $current = $ostatok->field_tp_kolichestvo->getString();
        $tovar_kolichestvo = $item->entity->get('field_tp_kolichestvo')->getString();

        if ($op) {
          $current = $current + $tovar_kolichestvo;
          $istoriya_op = '+';
        }
        else {
          $current = $current - $tovar_kolichestvo;
          $istoriya_op = '0';
        }

        \Drupal::service('trinion_tp.helper')->createDvigenieOstatokTovaraNode($ostatok, $tovar_kolichestvo, $node->id(), $edinica_izereniya_tid, $istoriya_op);
        $ostatok->field_tp_kolichestvo = $current;
        $ostatok->save();
      }
    }
  }

  /**
   * Обработка утверждения/аннулирования отгрузки
   * @param Node $node
   */
  public function utverzhdenieOtgruzki(Node $node, $op) {
    if ($zakaz_kilenta = $node->get('field_tp_zakaz_klienta')->first())
      $zakaz_kilenta = $zakaz_kilenta->entity;

    foreach ($node->get('field_tp_stroki') as $stroka_uit) {
      $stroka_uit = $stroka_uit->entity;
      $tovar_nid = $stroka_uit->get('field_tp_tovar')->getString();
      $sklad_tid = $stroka_uit->get('field_tp_sklad')->getString();
      $harakteristika_tid = $stroka_uit->get('field_tp_kharakteristika_tovara')->getString();
      $edinica_izereniya_tid = $stroka_uit->get('field_tp_edinica_izmereniya')->getString();
      $kolichestvo = $stroka_uit->get('field_tp_kolichestvo')->getString();
      $ostatok_node = \Drupal::service('trinion_tp.helper')->getOstatokTovaraNode($tovar_nid, $sklad_tid, FALSE, $harakteristika_tid, $edinica_izereniya_tid);

      if ($zakaz_kilenta && ($rezerv = \Drupal::service('trinion_tp.helper')->getRezervTovaraNodeByZakaz($zakaz_kilenta, $tovar_nid, $sklad_tid, $harakteristika_tid, $edinica_izereniya_tid))) {
        $rezerv_kolichestvo = $rezerv->get('field_tp_kolichestvo')->getString();
        $rezerv_kolichestvo_new = $rezerv_kolichestvo - $kolichestvo;
        if ($op) {
          $rezerv->field_tp_kolichestvo = $rezerv_kolichestvo_new > 0 ? $rezerv_kolichestvo_new : 0;
          $ostatok_node->field_tp_kolichestvo = $ostatok_node->get('field_tp_kolichestvo')->getString() - $kolichestvo;
          \Drupal::service('trinion_tp.helper')->createDvigenieOstatokTovaraNode($ostatok_node, $kolichestvo, $node->id(), $edinica_izereniya_tid, '-');
          \Drupal::messenger()->addStatus('Отгрузка утверждена');
        }
        else {
          $rezerv->field_tp_kolichestvo = $kolichestvo;
          $ostatok_node->field_tp_kolichestvo = $ostatok_node->get('field_tp_kolichestvo')->getString() + $kolichestvo;
          \Drupal::service('trinion_tp.helper')->createDvigenieOstatokTovaraNode($ostatok_node, $kolichestvo, $node->id(), $edinica_izereniya_tid, '0');
          \Drupal::messenger()->addStatus('Отгрузка аннулирована');
        }
        $rezerv->save();
      }
      $ostatok_node->save();
    }
  }

  /**
   * Обработка утверждения/аннулирования заказа клиента
   * @param Node $node
   * @param $op
   */
  public function utverzhdenieZakazaKlienta(Node $node, $op) {
    foreach ($node->get('field_tp_stroki') as $item) {
      $stroka = $item->entity;
      if ($tovar = $stroka->get('field_tp_tovar')->first()) {
        $tovar = $tovar->entity;
        $sklad = $item->entity->get('field_tp_sklad')->getString();
        $edinica_izereniya_tid = $item->entity->get('field_tp_edinica_izmereniya')->getString();
        $harakteristika_tid = $item->entity->get('field_tp_kharakteristika_tovara')->getString();
        switch ($op) {
          case 0:
            if ($rezerv = \Drupal::service('trinion_tp.helper')->getRezervTovaraNodeByZakaz($node, $tovar->id(), $sklad, $harakteristika_tid, $edinica_izereniya_tid))
              $rezerv->delete();
            break;
          case 1:
            $kolichestvo = $stroka->get('field_tp_kolichestvo')->getString();
            \Drupal::service('trinion_tp.helper')->createRezervTovaraNode($tovar, $sklad, $harakteristika_tid, $edinica_izereniya_tid, $kolichestvo, $node->id());
            break;
        }
      }
    }
  }
}
